==> vais_speech-1.0.8.tar.gz: ios/Classes/SwiftVaisSpeechPlugin.swift <==
import Flutter
import UIKit
import AVFoundation


class Queue<T> {
  private var elements: [T] = []

  func enqueue(_ value: T) {
    elements.append(value)
  }

  func dequeue() -> T? {
    guard !elements.isEmpty else {
      return nil
    }
    return elements.removeFirst()
  }

  func clear() {
    elements.removeAll()
   }

  var head: T? {
    return elements.first
  }

    var cout :Int? {
        return elements.count
    }

  var tail: T? {
    return elements.last
  }
}
@available(iOS 9.0, *)
public class SwiftVaisSpeechPlugin: NSObject, FlutterPlugin, AVAudioRecorderDelegate {
   // status - unset, initialized, recording, paused, stopped
      var status = "unset"
      var hasPermissions = false
      var mExtension = ""
      var mPath = ""
      var mSampleRate = 22050
      var bitRate = 128000
      var channel = 0
      var buffer:[UInt8] = []
      var shouldGetBuffer = false
      var startTime: Date!
      var UNUSAGED_VALUE = 0.0
      var isRecordRealtime = false

      var audioEngine: AVAudioEngine!
//      var inputNode : AVAudioInputNode!
      var outputFormat : AVAudioFormat!
    var isRecording = false
    var converterNode:AVAudioMixerNode!
      var aacFileURL:NSURL!
    //var queue : Queue<[String : Any]>!
    var outputAACFile : AVAudioFile!
    var audioRecorder: AVAudioRecorder?
    
      public static func register(with registrar: FlutterPluginRegistrar) {
          let channel = FlutterMethodChannel(name: "flutter_audio_recorder", binaryMessenger: registrar.messenger())
          let instance = SwiftVaisSpeechPlugin()
          registrar.addMethodCallDelegate(instance, channel: channel)
      }

      public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
          switch call.method {
          case "current":
            print("current")
           let dic = call.arguments as! [String : Any]
            channel = dic["channel"] as? Int ?? 0

            var recordingResult = [String : Any]()
            if(isRecordRealtime){
                recordingResult["duration"] = getDuration()
            }else{
                if(audioRecorder != nil){
                 recordingResult["duration"] =  Int(audioRecorder!.currentTime) * 1000
                }else{
                 recordingResult["duration"] =  0
                }
                
               
            }
            
            recordingResult["path"] = mPath
            recordingResult["audioFormat"] = mExtension
            recordingResult["peakPower"] = UNUSAGED_VALUE
            recordingResult["averagePower"] = UNUSAGED_VALUE
            recordingResult["isMeteringEnabled"] = false
            recordingResult["status"] = status
            recordingResult["iOSBuffer"] = buffer
            recordingResult["shouldGetBuffer"] = shouldGetBuffer
            shouldGetBuffer = false

            result(recordingResult)
            


             

          case "init":
              print("init")
              buffer.removeAll()

              let dic = call.arguments as! [String : Any]
              mExtension = dic["extension"] as? String ?? ".aac"
              mPath = dic["path"] as? String ?? ""
              isRecordRealtime = dic["realtime"] as? Bool ?? false
              bitRate = dic["bitRate"] as? Int ?? 128000
              
              if(isRecordRealtime){
                mSampleRate = 16000
                audioEngine = AVAudioEngine()
              }else{
                mSampleRate = 22050
              }
              
              
              print("m:", mExtension, mPath)
              print(mSampleRate)
              startTime = Date()
              if mPath == "" {
                  let documentsPath = NSSearchPathForDirectoriesInDomains(.documentDirectory, .userDomainMask, true)[0]
                  mPath = documentsPath + "/" + String(Int(startTime.timeIntervalSince1970)) + mExtension
                  print("path: " + mPath)
              }
              aacFileURL = NSURL(fileURLWithPath: mPath)

              
              if(isRecordRealtime){
                outputFormat = AVAudioFormat(commonFormat: .pcmFormatInt16,
                                                          sampleRate: Double(mSampleRate),
                                                          channels: AVAudioChannelCount(1),
                                                          interleaved: false)
              }
            status = "initialized"
            var recordingResult = [String : Any]()
            recordingResult["duration"] = 0
            recordingResult["path"] = mPath
            recordingResult["audioFormat"] = mExtension
            recordingResult["peakPower"] = UNUSAGED_VALUE
            recordingResult["averagePower"] = UNUSAGED_VALUE
            recordingResult["isMeteringEnabled"] = false
            recordingResult["status"] = status

            result(recordingResult)

          case "start":
              print("start")

              if status == "initialized" {
                if(isRecordRealtime){
                    resetEngineForRecord()
                    startEngine()
                }else{
                
                    stopRecording()
                    let settings = [
                        AVFormatIDKey: getOutputFormatFromString(mExtension),
                        AVEncoderBitRateKey: bitRate,
                        AVSampleRateKey: mSampleRate,
                        AVNumberOfChannelsKey: 2,
                        AVEncoderAudioQualityKey: AVAudioQuality.high.rawValue
                    ] as [String : Any]
                    
                    do {
                        try AVAudioSession.sharedInstance().setCategory(AVAudioSession.Category.playAndRecord, options: AVAudioSession.CategoryOptions.defaultToSpeaker)
                        try AVAudioSession.sharedInstance().setActive(true)
                    
                        audioRecorder = try AVAudioRecorder(url: URL(string: mPath)!, settings: settings)
                        audioRecorder!.delegate = self
                        audioRecorder!.record()
                        isRecording = true
                    } catch {
                        result(FlutterError(code: "", message: "Failed to start recording", details: nil))
                    }
                
                }
               
                status = "recording"
              }

              result(nil)

          case "stop":
              print("stop")

              status = "stopped"
              
              var recordingResult = [String : Any]()
              recordingResult["duration"] = 0
              recordingResult["path"] = mPath
              recordingResult["audioFormat"] = mExtension
              recordingResult["peakPower"] = UNUSAGED_VALUE
              recordingResult["averagePower"] = UNUSAGED_VALUE
              recordingResult["isMeteringEnabled"] = false
              recordingResult["status"] = status
              if(isRecordRealtime){
                audioEngine.inputNode.removeTap(onBus: 0)
                audioEngine.stop()
                audioEngine.reset()
              }else{
                stopRecording()
              }
              result(recordingResult)
          case "pause":
              print("pause")

              if status == "recording" {
                if(isRecordRealtime){
                    audioEngine.pause()
                }else{
                    audioRecorder!.pause()
                }
                  status = "paused"

              }

              result(nil)
          case "resume":
              print("resume")

              if status == "paused" {
                  do {
                    if(isRecordRealtime){
                        try audioEngine.start()
                    }else{
                         audioRecorder!.record()
                    }
                      
                      status = "recording"
                  } catch {
                      result(FlutterError(code: "", message: "Failed to resume", details: error))
                  }
              }

              result(nil)
          case "hasPermissions":
              print("hasPermissions")
              var permission: AVAudioSession.RecordPermission
              #if swift(>=4.2)
              permission = AVAudioSession.sharedInstance().recordPermission
              #else
              permission = AVAudioSession.sharedInstance().recordPermission()
              #endif

              switch permission {
              case .granted:
                  print("granted")
                  hasPermissions = true
                  result(hasPermissions)
                  break
              case .denied:
                  print("denied")
                  hasPermissions = false
                  result(hasPermissions)
                  break
              case .undetermined:
                  print("undetermined")

                  AVAudioSession.sharedInstance().requestRecordPermission() { [unowned self] allowed in
                      DispatchQueue.main.async {
                          if allowed {
                              self.hasPermissions = true
                              print("undetermined true")
                              result(self.hasPermissions)
                          } else {
                              self.hasPermissions = false
                              print("undetermined false")
                              result(self.hasPermissions)
                          }
                      }
                  }
                  break
              default:
                  result(hasPermissions)
                  break
              }
          default:
              result(FlutterMethodNotImplemented)
          }
      }


      // developer.apple.com/documentation/coreaudiotypes/coreaudiotype_constants/1572096-audio_data_format_identifiers
      func getOutputFormatFromString(_ format : String) -> Int {
          switch format {
          case ".mp4", ".aac", ".m4a":
              return Int(kAudioFormatMPEG4AAC)
          case ".wav":
              return Int(kAudioFormatLinearPCM)
          default :
              return Int(kAudioFormatMPEG4AAC)
          }
      }

    
      func getDuration() -> Int {
          do {
              let audioPlayer = try AVAudioPlayer(contentsOf: outputAACFile.url)
              print("Duration", Int(audioPlayer.duration))
            if(isRecordRealtime){
             return Int(audioPlayer.duration / 2) * 1000
            }else{
             return Int(audioPlayer.duration) * 1000
            }
          } catch {
              return -1
          }

      }
    fileprivate func stopRecording() {
      audioRecorder?.stop()
      audioRecorder = nil
      isRecording = false
    }

    


    private func resetEngineForRecord() {
        var settings = self.outputFormat.settings
        settings[AVFormatIDKey] = getOutputFormatFromString(self.mExtension)
        do {
            
            outputAACFile = try AVAudioFile(forWriting: self.aacFileURL! as URL,
                                            settings:settings)
            let audioSession = AVAudioSession.sharedInstance()
            try audioSession.setCategory(AVAudioSession.Category.playAndRecord)
            try audioSession.setActive(true)
            
            if(isRecordRealtime){
                audioEngine.inputNode.removeTap(onBus: 0)
                let input = audioEngine.inputNode
                let inputFormat = input.inputFormat(forBus: 0)
                let converter = AVAudioConverter(from: inputFormat, to: outputFormat!)!
                print("sample rate format")
                print(inputFormat.sampleRate)
                print(outputFormat.sampleRate)
                let ratio: Float = Float(inputFormat.sampleRate)/Float(outputFormat.sampleRate)

                input.installTap(onBus: 0, bufferSize: 1024, format: inputFormat) { (buffer, time) -> Void in
                    let inputCallback: AVAudioConverterInputBlock = { inNumPackets, outStatus in
                        outStatus.pointee = .haveData
                        return buffer
                    }

                    let convertedBuffer = AVAudioPCMBuffer(pcmFormat: self.outputFormat!, frameCapacity: UInt32(Float(buffer.frameCapacity) / ratio))!

                    var error: NSError?
                    let status = converter.convert(to: convertedBuffer, error: &error, withInputFrom: inputCallback)
                    assert(status != .error)

                    if (self.outputFormat?.commonFormat == AVAudioCommonFormat.pcmFormatInt16) {
                        let values = self.audioBufferToBytes(convertedBuffer)

                        self.buffer.removeAll()
                        self.buffer.append(contentsOf: values)
                        try! self.outputAACFile.write(from: buffer)
                        self.shouldGetBuffer = true


                       var recordingResult = [String : Any]()
                       recordingResult["duration"] = self.getDuration()
                       recordingResult["path"] = self.mPath
                      recordingResult["audioFormat"] = self.mExtension
                       recordingResult["peakPower"] = self.UNUSAGED_VALUE
                         recordingResult["averagePower"] = self.UNUSAGED_VALUE
                        recordingResult["isMeteringEnabled"] = false
                        recordingResult["status"] = self.status
                         recordingResult["iOSBuffer"] = values
                       recordingResult["shouldGetBuffer"] = self.shouldGetBuffer
                       //self.queue.enqueue(recordingResult)

                        //print("size queue")

                       //print(self.queue.cout!)
                    }
                }
            }


        } catch {
            print("fail")

        }

    }
    
    public func applicationWillTerminate(_ application: UIApplication) {
        if(!isRecordRealtime){
            print("applicationWillTerminate")
            stopRecording()
        }
       print("applicationWillTerminate")
    }
      
    public func applicationDidEnterBackground(_ application: UIApplication) {
      print("applicationDidEnterBackground")
       if(!isRecordRealtime){
            print("applicationDidEnterBackground")
            stopRecording()
        }
    }

    func copyAudioBufferBytes(_ audioBuffer: AVAudioPCMBuffer) -> [Int] {
        let srcLeft = audioBuffer.floatChannelData![0]

        let arrayFloat = Array(UnsafeBufferPointer(start: srcLeft, count: Int(audioBuffer.frameLength)))

        var arrayInt:[Int] = []
        for mFloat in arrayFloat {
            arrayInt.append(Int(mFloat * 32767))
        }
        // 2^15 - 1

        return arrayInt
      }

    private func audioBufferToBytes(_ audioBuffer: AVAudioPCMBuffer) -> [UInt8] {
        let srcLeft = audioBuffer.int16ChannelData![0]
        let bytesPerFrame = audioBuffer.format.streamDescription.pointee.mBytesPerFrame
        let numBytes = Int(bytesPerFrame * audioBuffer.frameLength)

        // initialize bytes by 0
        var audioByteArray = [UInt8](repeating: 0, count: numBytes)

        srcLeft.withMemoryRebound(to: UInt8.self, capacity: numBytes) { srcByteData in
            audioByteArray.withUnsafeMutableBufferPointer {
                $0.baseAddress!.initialize(from: srcByteData, count: numBytes)
            }
        }

        return audioByteArray
    }
    private func startEngine() {
        guard !audioEngine.isRunning else {
            return
        }

        try? audioEngine.start()
    }
}

==> vais_speech-2.0.1.tar.gz: ios/Classes/SwiftVaisSpeechPlugin.swift <==
import Flutter
import UIKit
import AVFoundation


class Queue<T> {
  private var elements: [T] = []

  func enqueue(_ value: T) {
    elements.append(value)
  }

  func dequeue() -> T? {
    guard !elements.isEmpty else {
      return nil
    }
    return elements.removeFirst()
  }

  func clear() {
    elements.removeAll()
   }

  var head: T? {
    return elements.first
  }

    var cout :Int? {
        return elements.count
    }

  var tail: T? {
    return elements.last
  }
}
@available(iOS 9.0, *)
public class SwiftVaisSpeechPlugin: NSObject, FlutterPlugin, AVAudioRecorderDelegate {
   // status - unset, initialized, recording, paused, stopped
      var status = "unset"
      var hasPermissions = false
      var mExtension = ""
      var mPath = ""
      var mSampleRate = 22050
      var bitRate = 128000
      var channel = 0
      var buffer:[UInt8] = []
      var shouldGetBuffer = false
      var startTime: Date!
      var UNUSAGED_VALUE = 0.0
      var isRecordRealtime = false

      var audioEngine: AVAudioEngine!
//      var inputNode : AVAudioInputNode!
      var outputFormat : AVAudioFormat!
    var isRecording = false
    var converterNode:AVAudioMixerNode!
      var aacFileURL:NSURL!
    //var queue : Queue<[String : Any]>!
    var outputAACFile : AVAudioFile!
    var audioRecorder: AVAudioRecorder?
    
      public static func register(with registrar: FlutterPluginRegistrar) {
          let channel = FlutterMethodChannel(name: "flutter_audio_recorder", binaryMessenger: registrar.messenger())
          let instance = SwiftVaisSpeechPlugin()
          registrar.addMethodCallDelegate(instance, channel: channel)

      }

      public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
          switch call.method {
          case "current":
            print("current")
           let dic = call.arguments as! [String : Any]
            channel = dic["channel"] as? Int ?? 0

            var recordingResult = [String : Any]()
            if(isRecordRealtime){
                recordingResult["duration"] = getDuration()
            }else{
                if(audioRecorder != nil){
                 recordingResult["duration"] =  Int(audioRecorder!.currentTime) * 1000
                }else{
                 recordingResult["duration"] =  0
                }
                
               
            }
            
            recordingResult["path"] = mPath
            recordingResult["audioFormat"] = mExtension
            recordingResult["peakPower"] = UNUSAGED_VALUE
            recordingResult["averagePower"] = UNUSAGED_VALUE
            recordingResult["isMeteringEnabled"] = false
            recordingResult["status"] = status
            recordingResult["iOSBuffer"] = buffer
            recordingResult["shouldGetBuffer"] = shouldGetBuffer
            shouldGetBuffer = false

            result(recordingResult)
            


             

          case "init":
              print("init")
              buffer.removeAll()
            print("nc.addObserver.")
            let nc = NotificationCenter.default
            nc.addObserver(self,
                           selector: #selector(handleInterruption),
                           name: AVAudioSession.interruptionNotification,
                           object: AVAudioSession.sharedInstance)
              let dic = call.arguments as! [String : Any]
              mExtension = dic["extension"] as? String ?? ".aac"
              mPath = dic["path"] as? String ?? ""
              isRecordRealtime = dic["realtime"] as? Bool ?? false
              bitRate = dic["bitRate"] as? Int ?? 128000
              
              if(isRecordRealtime){
                mSampleRate = 16000
                audioEngine = AVAudioEngine()
              }else{
                mSampleRate = 22050
              }
              
              
              print("m:", mExtension, mPath)
              print(mSampleRate)
              startTime = Date()
              if mPath == "" {
                  let documentsPath = NSSearchPathForDirectoriesInDomains(.documentDirectory, .userDomainMask, true)[0]
                  mPath = documentsPath + "/" + String(Int(startTime.timeIntervalSince1970)) + mExtension
                  print("path: " + mPath)
              }
              aacFileURL = NSURL(fileURLWithPath: mPath)

              
              if(isRecordRealtime){
                outputFormat = AVAudioFormat(commonFormat: .pcmFormatInt16,
                                                          sampleRate: Double(mSampleRate),
                                                          channels: AVAudioChannelCount(1),
                                                          interleaved: false)
              }
            status = "initialized"
            var recordingResult = [String : Any]()
            recordingResult["duration"] = 0
            recordingResult["path"] = mPath
            recordingResult["audioFormat"] = mExtension
            recordingResult["peakPower"] = UNUSAGED_VALUE
            recordingResult["averagePower"] = UNUSAGED_VALUE
            recordingResult["isMeteringEnabled"] = false
            recordingResult["status"] = status

            result(recordingResult)

          case "start":
              print("start")

              if status == "initialized" {
                if(isRecordRealtime){
                    resetEngineForRecord()
                    startEngine()
                }else{
                
                    stopRecording()
                    let settings = [
                        AVFormatIDKey: getOutputFormatFromString(mExtension),
                        AVEncoderBitRateKey: bitRate,
                        AVSampleRateKey: mSampleRate,
                        AVNumberOfChannelsKey: 2,
                        AVEncoderAudioQualityKey: AVAudioQuality.high.rawValue
                    ] as [String : Any]
                    
                    do {
                        try AVAudioSession.sharedInstance().setCategory(AVAudioSession.Category.playAndRecord, options: [.defaultToSpeaker,.mixWithOthers])
//                        try AVAudioSession.sharedInstance().setCategory(AVAudioSession.Category.playback, options: AVAudioSession.CategoryOptions.mixWithOthers)
                        try AVAudioSession.sharedInstance().setActive(true)
                
                        audioRecorder = try AVAudioRecorder(url: URL(string: mPath)!, settings: settings)
                        audioRecorder!.delegate = self
                        audioRecorder!.record()
                        isRecording = true
                    } catch {
                        result(FlutterError(code: "", message: "Failed to start recording", details: nil))
                    }
                
                }
               
                status = "recording"
              }

              result(nil)

          case "stop":
              print("stop")

              status = "stopped"
              
              var recordingResult = [String : Any]()
              recordingResult["duration"] = 0
              recordingResult["path"] = mPath
              recordingResult["audioFormat"] = mExtension
              recordingResult["peakPower"] = UNUSAGED_VALUE
              recordingResult["averagePower"] = UNUSAGED_VALUE
              recordingResult["isMeteringEnabled"] = false
              recordingResult["status"] = status
              if(isRecordRealtime){
                audioEngine.inputNode.removeTap(onBus: 0)
                audioEngine.stop()
                audioEngine.reset()
              }else{
                stopRecording()
              }
              result(recordingResult)
          case "pause":
              print("pause")

              if status == "recording" {
                if(isRecordRealtime){
                    audioEngine.pause()
                }else{
                    audioRecorder!.pause()
                }
                  status = "paused"

              }

              result(nil)
          case "resume":
              print("resume")

              if status == "paused" {
                  do {
                    if(isRecordRealtime){
                        try audioEngine.start()
                    }else{
                         audioRecorder!.record()
                    }
                      
                      status = "recording"
                  } catch {
                      result(FlutterError(code: "", message: "Failed to resume", details: error))
                  }
              }

              result(nil)
          case "hasPermissions":
              print("hasPermissions")
              var permission: AVAudioSession.RecordPermission
              #if swift(>=4.2)
              permission = AVAudioSession.sharedInstance().recordPermission
              #else
              permission = AVAudioSession.sharedInstance().recordPermission()
              #endif

              switch permission {
              case .granted:
                  print("granted")
                  hasPermissions = true
                  result(hasPermissions)
                  break
              case .denied:
                  print("denied")
                  hasPermissions = false
                  result(hasPermissions)
                  break
              case .undetermined:
                  print("undetermined")

                  AVAudioSession.sharedInstance().requestRecordPermission() { [unowned self] allowed in
                      DispatchQueue.main.async {
                          if allowed {
                              self.hasPermissions = true
                              print("undetermined true")
                              result(self.hasPermissions)
                          } else {
                              self.hasPermissions = false
                              print("undetermined false")
                              result(self.hasPermissions)
                          }
                      }
                  }
                  break
              default:
                  result(hasPermissions)
                  break
              }
          default:
              result(FlutterMethodNotImplemented)
          }
      }
    


      @objc func handleInterruption(notification: Notification) {
        print("handleInterruption.")
        guard let userInfo = notification.userInfo,
            let typeValue = userInfo[AVAudioSessionInterruptionTypeKey] as? UInt,
            let type = AVAudioSession.InterruptionType(rawValue: typeValue) else {
                return
        }

        // Switch over the interruption type.
        switch type {

        case .began:
            print("began.")
            break
            // An interruption began. Update the UI as necessary.

        case .ended:
           // An interruption ended. Resume playback, if appropriate.
            print("ended.")
            guard let optionsValue = userInfo[AVAudioSessionInterruptionOptionKey] as? UInt else { return }
            let options = AVAudioSession.InterruptionOptions(rawValue: optionsValue)
            if options.contains(.shouldResume) {
                // An interruption ended. Resume playback.
                print("An interruption ended. Resume playback.")
            } else {
                // An interruption ended. Don't resume playback.
                print("An interruption ended. Don't resume playback.")
            }
            break

        default: ()
        }
      }
    
      // developer.apple.com/documentation/coreaudiotypes/coreaudiotype_constants/1572096-audio_data_format_identifiers
      func getOutputFormatFromString(_ format : String) -> Int {
          switch format {
          case ".mp4", ".aac", ".m4a":
              return Int(kAudioFormatMPEG4AAC)
          case ".wav":
              return Int(kAudioFormatLinearPCM)
          default :
              return Int(kAudioFormatMPEG4AAC)
          }
      }

    
      func getDuration() -> Int {
          do {
              let audioPlayer = try AVAudioPlayer(contentsOf: outputAACFile.url)
              print("Duration", Int(audioPlayer.duration))
            if(isRecordRealtime){
             return Int(audioPlayer.duration / 2) * 1000
            }else{
             return Int(audioPlayer.duration) * 1000
            }
          } catch {
              return -1
          }

      }
    fileprivate func stopRecording() {
      audioRecorder?.stop()
      audioRecorder = nil
      isRecording = false
    }

    


    private func resetEngineForRecord() {
        var settings = self.outputFormat.settings
        settings[AVFormatIDKey] = getOutputFormatFromString(self.mExtension)
        do {
            
            outputAACFile = try AVAudioFile(forWriting: self.aacFileURL! as URL,
                                            settings:settings)
            let audioSession = AVAudioSession.sharedInstance()
            try audioSession.setCategory(AVAudioSession.Category.playAndRecord)
            try audioSession.setActive(true)
            
            if(isRecordRealtime){
                audioEngine.inputNode.removeTap(onBus: 0)
                let input = audioEngine.inputNode
                let inputFormat = input.inputFormat(forBus: 0)
                let converter = AVAudioConverter(from: inputFormat, to: outputFormat!)!
                print("sample rate format")
                print(inputFormat.sampleRate)
                print(outputFormat.sampleRate)
                let ratio: Float = Float(inputFormat.sampleRate)/Float(outputFormat.sampleRate)

                input.installTap(onBus: 0, bufferSize: 1024, format: inputFormat) { (buffer, time) -> Void in
                    let inputCallback: AVAudioConverterInputBlock = { inNumPackets, outStatus in
                        outStatus.pointee = .haveData
                        return buffer
                    }

                    let convertedBuffer = AVAudioPCMBuffer(pcmFormat: self.outputFormat!, frameCapacity: UInt32(Float(buffer.frameCapacity) / ratio))!

                    var error: NSError?
                    let status = converter.convert(to: convertedBuffer, error: &error, withInputFrom: inputCallback)
                    assert(status != .error)

                    if (self.outputFormat?.commonFormat == AVAudioCommonFormat.pcmFormatInt16) {
                        let values = self.audioBufferToBytes(convertedBuffer)

                        self.buffer.removeAll()
                        self.buffer.append(contentsOf: values)
                        try! self.outputAACFile.write(from: buffer)
                        self.shouldGetBuffer = true


                       var recordingResult = [String : Any]()
                       recordingResult["duration"] = self.getDuration()
                       recordingResult["path"] = self.mPath
                      recordingResult["audioFormat"] = self.mExtension
                       recordingResult["peakPower"] = self.UNUSAGED_VALUE
                         recordingResult["averagePower"] = self.UNUSAGED_VALUE
                        recordingResult["isMeteringEnabled"] = false
                        recordingResult["status"] = self.status
                         recordingResult["iOSBuffer"] = values
                       recordingResult["shouldGetBuffer"] = self.shouldGetBuffer
                       //self.queue.enqueue(recordingResult)

                        //print("size queue")

                       //print(self.queue.cout!)
                    }
                }
            }


        } catch {
            print("fail")

        }

    }
    
    public func applicationWillTerminate(_ application: UIApplication) {
        if(!isRecordRealtime){
            print("applicationWillTerminate")
            stopRecording()
        }
       print("applicationWillTerminate")
    }
      
    public func applicationDidEnterBackground(_ application: UIApplication) {
      print("applicationDidEnterBackground")
       if(!isRecordRealtime){
            print("applicationDidEnterBackground")
            stopRecording()
        }
    }

    func copyAudioBufferBytes(_ audioBuffer: AVAudioPCMBuffer) -> [Int] {
        let srcLeft = audioBuffer.floatChannelData![0]

        let arrayFloat = Array(UnsafeBufferPointer(start: srcLeft, count: Int(audioBuffer.frameLength)))

        var arrayInt:[Int] = []
        for mFloat in arrayFloat {
            arrayInt.append(Int(mFloat * 32767))
        }
        // 2^15 - 1

        return arrayInt
      }

    private func audioBufferToBytes(_ audioBuffer: AVAudioPCMBuffer) -> [UInt8] {
        let srcLeft = audioBuffer.int16ChannelData![0]
        let bytesPerFrame = audioBuffer.format.streamDescription.pointee.mBytesPerFrame
        let numBytes = Int(bytesPerFrame * audioBuffer.frameLength)

        // initialize bytes by 0
        var audioByteArray = [UInt8](repeating: 0, count: numBytes)

        srcLeft.withMemoryRebound(to: UInt8.self, capacity: numBytes) { srcByteData in
            audioByteArray.withUnsafeMutableBufferPointer {
                $0.baseAddress!.initialize(from: srcByteData, count: numBytes)
            }
        }

        return audioByteArray
    }
    private func startEngine() {
        guard !audioEngine.isRunning else {
            return
        }

        try? audioEngine.start()
    }
}
